#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import sqlite3
import os
import time
import json
import threading

import manuf as mf
import tools

def sqlConnection(db_file):
    if not os.path.isfile(db_file):
        con = createDB(db_file)
    else:
        con = sqlite3.connect(db_file)
    return con

def createDB(db_file):

    con = sqlite3.connect(db_file)
    cur = con.cursor()

    cur.execute('''CREATE TABLE IF NOT EXISTS arp (mac TEXT PRIMARY KEY NOT NULL,ip TEXT,data TEXT);''')
    cur.execute('''CREATE UNIQUE INDEX IF NOT EXISTS idx_mac ON arp (mac);''')

    cur.execute('''CREATE TABLE IF NOT EXISTS ports (port INTEGER PRIMARY KEY NOT NULL,data TEXT);''')
    cur.execute('''CREATE UNIQUE INDEX IF NOT EXISTS idx_port ON ports (port);''')

    create_established =  "CREATE TABLE IF NOT EXISTS established (rule TEXT CHECK(rule IN ('ALLOW','DENY')) NOT NULL DEFAULT 'ALLOW',"
    create_established += "proto TEXT,laddr TEXT,lport INTEGER,faddr TEXT,fport INTEGER,UNIQUE(rule,proto,laddr,lport,faddr,fport));"
    cur.execute(create_established)

    create_nmap  = "CREATE TABLE IF NOT EXISTS nmap (ip TEXT PRIMARY KEY NOT NULL,timestamp TEXT,data TEXT);"
    create_nmapi = "CREATE UNIQUE INDEX IF NOT EXISTS idx_nmap ON nmap (ip);"
    cur.execute(create_nmap)
    cur.execute(create_nmapi)

    create_vulns  = "CREATE TABLE IF NOT EXISTS vulns (vid INTEGER PRIMARY KEY NOT NULL,ip TEXT,timestamp TEXT,report TEXT,data TEXT,blob BLOB);"
    create_vulnsi = "CREATE UNIQUE INDEX IF NOT EXISTS idx_vulns ON vulns (vid);"
    cur.execute(create_vulns)
    cur.execute(create_vulnsi)

    create_detect  = "CREATE TABLE IF NOT EXISTS detect (did INTEGER PRIMARY KEY NOT NULL,ip TEXT,timestamp TEXT,report TEXT,data TEXT,blob BLOB);"
    create_detecti = "CREATE UNIQUE INDEX IF NOT EXISTS idx_detect ON detect (did);"
    cur.execute(create_detect)
    cur.execute(create_detecti)

    #create_ip  = "CREATE TABLE IF NOT EXISTS ips (name TEXT PRIMARY KEY NOT NULL,timestamp TEXT,data TEXT);"
    create_ip  = "CREATE TABLE IF NOT EXISTS ips (name TEXT PRIMARY KEY NOT NULL,timestamp TEXT,data JSON) WITHOUT ROWID;"
    create_ipi = "CREATE UNIQUE INDEX IF NOT EXISTS idx_ip ON ips (name);"
    cur.execute(create_ip)
    cur.execute(create_ipi)

    #create_configs  = "CREATE TABLE IF NOT EXISTS configs (name TEXT PRIMARY KEY NOT NULL,timestamp TEXT,data TEXT);"
    create_configs  = "CREATE TABLE IF NOT EXISTS configs (name TEXT PRIMARY KEY NOT NULL,timestamp TEXT,data JSON) WITHOUT ROWID;"
    create_configsi = "CREATE UNIQUE INDEX IF NOT EXISTS idx_config ON configs (name);"
    cur.execute(create_configs)
    cur.execute(create_configsi)

    #create_fims  = "CREATE TABLE IF NOT EXISTS fims (name TEXT PRIMARY KEY NOT NULL,timestamp TEXT,data TEXT);"
    create_fims  = "CREATE TABLE IF NOT EXISTS fims (name TEXT PRIMARY KEY NOT NULL, timestamp TEXT, data JSON) WITHOUT ROWID;"
    create_fimsi = "CREATE UNIQUE INDEX IF NOT EXISTS idx_fims ON fims (name);"
    cur.execute(create_fims)
    cur.execute(create_fimsi)

    create_files  = "CREATE TABLE IF NOT EXISTS files (name TEXT PRIMARY KEY NOT NULL, timestamp TEXT, data BLOB) WITHOUT ROWID;"
    create_filesi = "CREATE UNIQUE INDEX IF NOT EXISTS idx_files ON files (name);"
    cur.execute(create_files)
    cur.execute(create_filesi)

    #create_reports  = "CREATE TABLE IF NOT EXISTS reports (name TEXT PRIMARY KEY NOT NULL,timestamp TEXT,data TEXT);"
    create_reports  = "CREATE TABLE IF NOT EXISTS reports (name TEXT PRIMARY KEY NOT NULL,timestamp TEXT,data JSON) WITHOUT ROWID;"
    create_reportsi = "CREATE UNIQUE INDEX IF NOT EXISTS idx_reports ON reports (name);"
    cur.execute(create_reports)
    cur.execute(create_reportsi)

    #create_alerts  = "CREATE TABLE IF NOT EXISTS alerts (name TEXT PRIMARY KEY NOT NULL,timestamp TEXT,data JSON,status INT,run JSON);"
    create_alerts  = "CREATE TABLE IF NOT EXISTS alerts (name TEXT PRIMARY KEY NOT NULL,timestamp TEXT,data JSON) WITHOUT ROWID;"
    create_alertsi = "CREATE UNIQUE INDEX IF NOT EXISTS idx_alerts ON alerts (name);"
    cur.execute(create_alerts)
    cur.execute(create_alertsi)

    #create_jobs  = "CREATE TABLE IF NOT EXISTS jobs (name TEXT PRIMARY KEY NOT NULL,timestamp TEXT,data TEXT);"
    create_jobs  = "CREATE TABLE IF NOT EXISTS jobs (name TEXT PRIMARY KEY NOT NULL,timestamp TEXT,data JSON) WITHOUT ROWID;"
    create_jobsi = "CREATE UNIQUE INDEX IF NOT EXISTS idx_jobs ON jobs (name);"
    cur.execute(create_jobs)
    cur.execute(create_jobsi)

    create_counts  = "CREATE TABLE IF NOT EXISTS counts (name TEXT PRIMARY KEY NOT NULL,count INTEGER) WITHOUT ROWID;"
    create_countsi = "CREATE UNIQUE INDEX IF NOT EXISTS idx_counts ON counts (name);"
    cur.execute(create_counts)
    cur.execute(create_countsi)

    create_proms  = "CREATE TABLE IF NOT EXISTS proms (name TEXT PRIMARY KEY NOT NULL,data TEXT) WITHOUT ROWID;"
    create_promsi = "CREATE UNIQUE INDEX IF NOT EXISTS idx_proms ON proms (name);"
    cur.execute(create_proms)
    cur.execute(create_promsi)

    create_b2sums  = "CREATE TABLE IF NOT EXISTS b2sum (name TEXT PRIMARY KEY NOT NULL,data JSON) WITHOUT ROWID;"
    create_b2sumsi = "CREATE UNIQUE INDEX IF NOT EXISTS idx_b2sum ON b2sum (name);"
    cur.execute(create_b2sums)
    cur.execute(create_b2sumsi)

    #create_sshwatch  = "CREATE TABLE IF NOT EXISTS sshwatch (name TEXT PRIMARY KEY NOT NULL,data JSON) WITHOUT ROWID;"
    #create_sshwatchi = "CREATE UNIQUE INDEX IF NOT EXISTS idx_sshwatch ON sshwatch (name);"
    #cur.execute(create_sshwatch)
    #cur.execute(create_sshwatchi)

    create_rules  = "CREATE TABLE IF NOT EXISTS rules (name TEXT PRIMARY KEY NOT NULL,timestamp TEXT,data JSON) WITHOUT ROWID;"
    create_rulesi = "CREATE UNIQUE INDEX IF NOT EXISTS idx_rules ON rules (name);"
    cur.execute(create_rules)
    cur.execute(create_rulesi)

    create_model  = "CREATE TABLE IF NOT EXISTS model (tag TEXT, data JSON);"
    create_modeli = "CREATE UNIQUE INDEX IF NOT EXISTS idx_model ON model (tag, data);"
    cur.execute(create_model)
    cur.execute(create_modeli)

    create_training  = "CREATE TABLE IF NOT EXISTS training (tag TEXT, data JSON);"
    create_trainingi = "CREATE UNIQUE INDEX IF NOT EXISTS idx_training ON training (tag, data);"
    cur.execute(create_training)
    cur.execute(create_trainingi)

    create_occurrence  = "CREATE TABLE IF NOT EXISTS occurrence (name TEXT PRIMARY KEY NOT NULL, tag INTEGER, data JSON) WITHOUT ROWID;"
    create_occurrencei = "CREATE UNIQUE INDEX IF NOT EXISTS idx_occurrence ON occurrence (name);"
    cur.execute(create_occurrence)
    cur.execute(create_occurrencei)

    create_system_profile  = "CREATE TABLE IF NOT EXISTS system_profile (name TEXT PRIMARY KEY NOT NULL, timestamp TEXT, data JSON);"
    create_system_profilei = "CREATE UNIQUE INDEX IF NOT EXISTS idx_system_profile ON system_profile (name);"
    cur.execute(create_system_profile)
    cur.execute(create_system_profilei)

    create_bearer_token = "CREATE TABLE IF NOT EXISTS bearer (token TEXT PRIMARY KEY NOT NULL, timestamp TEXT, data JSON);"
    create_bearer_tokeni = "CREATE UNIQUE INDEX IF NOT EXISTS idx_bearer_token ON bearer (token);"
    cur.execute(create_bearer_token)
    cur.execute(create_bearer_tokeni)

    # https://www.sqlite.org/autoinc.html
    # Whenever you create a table without specifying the WITHOUT ROWID option,
    # you get an implicit auto-increment column called rowid. 
    # The rowid column store 64-bit signed integer that uniquely identifies a row in the table.
    #create_client_commands  = "CREATE TABLE IF NOT EXISTS client_commands (id INTEGER PRIMARY KEY NOT NULL, token TEXT, timestamp TEXT, data JSON);"
    create_client_commands  = "CREATE TABLE IF NOT EXISTS client_commands (token TEXT, timestamp TEXT, data JSON);"
    #create_client_commandsi = "CREATE UNIQUE INDEX IF NOT EXISTS idx_client_commands ON client_commands (rowid);"
    cur.execute(create_client_commands)
    #cur.execute(create_client_commandsi)


    con.commit()

    return con

###############################################################################################################################################

class DNSUpDateTask:
    def __init__(self):
        self._running = True

    def terminate(self):
        self._running = False

    def sqlConnection(self, db_file):
        con = sqlite3.connect(db_file)
        return con

    def run(self, mac, ip, db_file):
        #print(mac, ip, db_file)
        ip = ip.strip('(')
        ip = ip.strip(')')
        #print('IP: ' + ip)

        #dnsname = tools.getDNSName(ip)
        dnsname = str(tools.getNSlookup(ip))

        #print('DNS: ' + dnsname)
        con = self.sqlConnection(db_file)
        cur = con.cursor()
        cur.execute("SELECT data FROM arp WHERE mac=?", (mac,))
        record = cur.fetchone()
        if record is None:
            return False
        #print(record[0])
        jdata = json.loads(record[0])
        jdata['dns'] = dnsname
        update = json.dumps(jdata)
        cur.execute("UPDATE arp SET data=? WHERE mac=?", (update, mac))
        con.commit()
        print('updated dns ' + str(mac) + ' ' + str(update))
        if cur.rowcount == 0:
            return False
        return True


def update_arp_data(db_file, arpDict, manuf_file):

    con = sqlConnection(db_file)
    cur = con.cursor()

    for ip,mac in arpDict.items():

        #if mac == '(incomplete)':
        if (mac == '(incomplete)') or (mac == '<incomplete>'):
            #print('SKIP (incomplete) ' + ip)
            #check if leftover ip,
            cur.execute("SELECT mac,ip FROM arp WHERE ip LIKE '%" + ip + "%'")
            rows = cur.fetchall()
            for row in rows:
                _mac = row[0]
                line = row[1].split(',')
                #print('Match.This.Row ' + str(line))
                line.remove(ip) #remove doesn't return anything, it modifies existing list in place
                #print('new list ', line)
              
                l = ''
                c = len(line)
                for i in line:
                    #print(c, i)
                    if c == 1:
                        l = l + i
                    else:
                        l =  i + ',' + l
                    c -= 1

                #print(l)
                cur.execute("UPDATE arp SET ip=? WHERE mac=?", (l, _mac))
                con.commit()
                print('updated.2 ' + str(_mac) + ' ' + str(l))
            continue #print('SKIP (incomplete)')

        cur.execute("SELECT ip FROM arp WHERE mac='" + mac + "'")
        #_mac, _ip = cur.fetchone()
        _result = cur.fetchone()
        #print(_mac, _ip, _data)
        if not _result:
            t = time.strftime("%Y-%m-%dT%H:%M:%SZ")
            #m = mf.get_manuf(mac, 'db/manuf')
            m = mf.get_manuf(mac, manuf_file)
            #print(m)
            data = '{"created":"' + t + '","manuf":"' + m + '"}'
            cur.execute("INSERT INTO arp VALUES (?, ?, ?)", (mac, ip, data))
            con.commit()
            print('new ' + str(mac) + ' ' + str(ip) + ' ' + str(data))
            # launch dns thread update async
            dns = DNSUpDateTask()
            t = threading.Thread(target=dns.run, args=(mac,ip,db_file,))
            #print('t.start')
            t.start()
        else:
            #print(ip, _result[0]) #tuple _result
            if ip not in _result[0]:
                #print(len(_result[0]))
                if len(_result[0]) == 0:
                    _ip = ip + _result[0]
                else:
                    _ip = ip + ',' + _result[0] #csv

                #if len(_result[0]) > 0:
                #    _ip = ip + ',' + _result[0] #csv
                #else:
                #    _ip = ip + _result[0]

                cur.execute("UPDATE arp SET ip=? WHERE mac=?", (_ip, mac))
                con.commit()
                print('updated.1 ' + str(mac) + ' ' + str(_ip))
    return True


def select_all(db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute('SELECT * FROM arp')
    rows = cur.fetchall()
    return rows


def print_all(db_file):
    rows = select_all(db_file)
    for row in rows:
        print(row)
    return True


#def insert_table(con):
#    cur = con.cursor()
#    cur.execute("INSERT INTO arp VALUES('ff:ff:ff:ff:ff:ff', '(192.168.0.255)', '{}')")
#    con.commit()
#    return True


def update_data_manuf(mac, mfname, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    #cur.execute("SELECT data FROM arp WHERE mac='" + str(mac) + "';")
    cur.execute("SELECT data FROM arp WHERE mac=?", (mac,))
    record = cur.fetchone()
    if record is None:
        return None
    #print(record[0])
    jdata = json.loads(record[0])
    jdata['manuf'] = mfname
    #print(json.dumps(jdata))
    update = json.dumps(jdata)
    cur.execute("UPDATE arp SET data=? WHERE mac=?", (update, mac))
    con.commit()
    print('updated.manuf ' + str(mac) + ' ' + str(update))
    return True

def update_data_dns(mac, dnsname, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute("SELECT data FROM arp WHERE mac=?", (mac,))
    record = cur.fetchone()
    if record is None:
        return None
    #print(record[0])
    jdata = json.loads(record[0])
    jdata['dns'] = dnsname
    #print(json.dumps(jdata))
    update = json.dumps(jdata)
    cur.execute("UPDATE arp SET data=? WHERE mac=?", (update, mac))
    con.commit()
    print('updated.dns ' + str(mac) + ' ' + str(update))
    return True

def get_data(mac, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute("SELECT data FROM arp WHERE mac=?", (mac,))
    record = cur.fetchone()
    if record is None:
        return None
    #print(record[0])
    jdata = json.loads(record[0])
    return jdata


def get_manuf(mac, manuf_file):
    #manuf = mf.get_manuf(mac, 'db/manuf')
    manuf = mf.get_manuf(mac, manuf_file)
    return manuf

def get_bearer(token, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute("SELECT token,data FROM bearer WHERE token=?", (token,))
    record = cur.fetchone()
    if record is None:
        return None
    return record

    #print(record[0])
    #jdata = json.loads(record[0])
    #return jdata


#def update_manuf(mac, manuf_file, db_file):
#    mfname = get_manuf(mac, manuf_file)
#    update = update_data_manuf(mac, mfname, db_file)
#    return update

def printListeningAllowed(db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute('SELECT * FROM ports')
    rows = cur.fetchall()
    for row in rows:
        print(row)
    return True

def gettListeningAllowedLst(db_file):
    portLst = []
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute('SELECT port FROM ports')
    rows = cur.fetchall()
    for row in rows:
        #print(row)
        _row = row[0]
        portLst.append(_row)
    return portLst


def insertAllowedPort(port, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute("INSERT INTO ports VALUES(?,?)", (port, '{}'))
    con.commit()
    return True

def deleteAllowedPort(port, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute("DELETE FROM ports WHERE port=?", (port,))
    con.commit()
    return True

def getListenPortsLst():
    open_portsLst = []
    open_portsDct = tools.getListenPortsDct()
    for open_port in open_portsDct:
        open_portsLst.append(open_port)
    return open_portsLst

def printListeningAlerts(db_file):

    open_portsLst = getListenPortsLst()
    allow_portsLst = gettListeningAllowedLst(db_file)

    diffLst = list(set(open_portsLst) - set(allow_portsLst))
    print(sorted(diffLst))

    return True
    #tools.listenPortsLst() #unsorted list tcp4:631, tcp6:631


def getEstablishedRulesDct(db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute('SELECT * FROM established')
    rows = cur.fetchall()

    Dct = {}
    c = 0
    for row in rows:
        #print(row)
        c += 1
        Dct[c] = row
        #_row = row[0]
        #portLst.append(_row)
    #return portLst
    #return True
    return Dct

def insertEstablishedRules(rule, proto, laddr, lport, faddr, fport, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute("INSERT INTO established VALUES(?,?,?,?,?,?)", (rule, proto, laddr, lport, faddr, fport))
    con.commit()
    return True

def insertIPs(ip, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    #cur.execute("INSERT INTO ips VALUES(?, DATETIME('now'), ?)", (ip, None ))
    cur.execute("INSERT INTO ips VALUES(?, DATETIME('now'), NULL)", (ip, ))
    con.commit()
    return True

def updateIPs(ip, data, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    #print('UPDATE ' + ip + ' ' + data)
    #cur.execute("UPDATE ips SET data=? WHERE ip=?", (ip, data))
    sql = "UPDATE ips SET data='" + data + "', timestamp=DATETIME('now') WHERE ip='" + ip + "';"
    cur.execute(sql)
    con.commit()
    return True

def replaceIPs(ip, data, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute("REPLACE INTO ips VALUES(?, DATETIME('now'), ?)", (ip, data))
    con.commit()
    return True

#def clearAllIPs(db_file):
#    con = sqlConnection(db_file)
#    cur = con.cursor()
#    cur.execute("DELETE FROM ips;")
#    cur.execute("REINDEX ips;")
#    con.commit()
#    return True

def deleteIPs(ip, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    #cur.execute("DELETE FROM ips WHERE ip=? ;", (ip,))
    cur.execute("DELETE FROM ips WHERE name=? ;", (ip,))
    rowid = cur.lastrowid
    rowcount = cur.rowcount
    con.commit()
    #if cur.rowcount == 0:
    #    return None
    #return rowid
    #return True

    if rowcount == 1:
        return True
    else:
        return False
    #return rowcount

def getNmaps(db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute('SELECT rowid,* FROM nmap ORDER by rowid DESC;')
    rows = cur.fetchall()
    return rows

def replaceNmaps(ip, data, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute("REPLACE INTO nmap VALUES(?, DATETIME('now'), ?)", (ip, data))
    con.commit()
    return True

def deleteNmaps(ip, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute("DELETE FROM nmap WHERE ip=? ;", (ip,))
    con.commit()
    return True

def clearAllNmaps(db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute("DELETE FROM nmap;")
    cur.execute("REINDEX nmap;")
    con.commit()
    return True

def getNmapVuln(vid, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute('SELECT rowid,* FROM vulns WHERE rowid=? ;', (vid,))
    row = cur.fetchone()
    return row

def getAllNmapVulns(db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute('SELECT rowid,* FROM vulns ORDER by rowid DESC;')
    rows = cur.fetchall()
    return rows

def replaceVulns(ip, data, blob, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute("REPLACE INTO vulns VALUES(?,DATETIME('now'),?,?)", (ip, data, blob))
    con.commit()
    return True

def clearAllVulns(db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute("DELETE FROM vulns;")
    cur.execute("REINDEX vulns;")
    con.commit()
    return True

def deleteVulns(rowid, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute("DELETE FROM vulns WHERE rowid=? ;", (rowid,))
    con.commit()
    return True

def insertVulns(ip, report, data, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute("INSERT INTO vulns VALUES(NULL,?,DATETIME('now'),?,?,NULL)", (ip,report,data))
    con.commit()
    return True

def updateVulnsReport(vid, report, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    sql = "UPDATE vulns SET report='" + report + "' WHERE vid='" + str(vid) + "';"
    cur.execute(sql)
    con.commit()
    return True

def getVulnData(vid, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute('SELECT data FROM vulns WHERE rowid=? ;', (vid,))
    row = cur.fetchone()
    return row

def getNmapDetect(did, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute('SELECT rowid,* FROM detect WHERE rowid=? ;', (did,))
    row = cur.fetchone()
    return row

def getAllNmapDetects(db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute('SELECT rowid,* FROM detect ORDER by rowid DESC;')
    rows = cur.fetchall()
    return rows

def insertDetect(ip, report, data, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute("INSERT INTO detect VALUES(NULL,?,DATETIME('now'),?,?,NULL)", (ip,report,data))
    con.commit()
    return True

def deleteDetect(rowid, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute("DELETE FROM detect WHERE rowid=? ;", (rowid,))
    con.commit()
    return True

def clearAllDetects(db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute("DELETE FROM detect;")
    cur.execute("REINDEX detect;")
    con.commit()
    return True

#def getAllConfigs(db_file):
#    con = sqlConnection(db_file)
#    cur = con.cursor()
#    cur.execute('SELECT rowid,* FROM configs;')
#    rows = cur.fetchall()
#    return rows

#def getAllFims(db_file):
#    rows = getAll('fims', db_file)
#    return rows


def clearAll(tbl, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute('DELETE FROM ' + str(tbl) + ';')
    cur.execute('REINDEX ' + str(tbl) + ';')
    con.commit()
    return True


def getConfig(name, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute('SELECT data FROM configs WHERE name=? ;', (name,))
    row = cur.fetchone()
    return row

def getFim(name, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute('SELECT data FROM fims WHERE name=? ;', (name,))
    row = cur.fetchone()
    if cur.rowcount == 0:
        return None
    return row

def getData(tbl, name, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute('SELECT data FROM ' + str(tbl) + ' WHERE name=? ;', (name,))
    row = cur.fetchone()
    if cur.rowcount == 0:
        return None
    return row


#def replaceFim(name, data, db_file):
#    con = sqlConnection(db_file)
#    cur = con.cursor()
#    cur.execute("REPLACE INTO fims VALUES(?,DATETIME('now'),?)", (name, data))
#    con.commit()
#    return True


def insertConfig(name, data, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute("INSERT INTO configs VALUES(?,DATETIME('now'),?)", (name,data))
    con.commit()
    return True

def replaceConfig(name, data, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute("REPLACE INTO configs VALUES(?,DATETIME('now'),?)", (name, data))
    con.commit()
    if cur.rowcount == 0:
        return False
    return True

def replaceINTOproms(name, data, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute("REPLACE INTO proms VALUES(?,?)", (name, data))
    con.commit()
    if cur.rowcount == 0:
        return False
    return True

#def replaceINTO2(tbl, name, data, db_file):
def replaceINTOduce(tbl, name, data, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute('REPLACE INTO ' + str(tbl) + ' VALUES(?,?)', (name, data))
    con.commit()
    if cur.rowcount == 0:
        return False
    return True

def replaceINTOducedate(tbl, name, data, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute("REPLACE INTO " + str(tbl) + " VALUES(?,DATETIME('now'),?)", (name, data))
    con.commit()
    if cur.rowcount == 0:
        return False
    return True


def replaceINTOtrio(tbl, name, tag, data, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute('REPLACE INTO ' + str(tbl) + ' VALUES(?,?,?)', (name, tag, data))
    con.commit()
    if cur.rowcount == 0:
        return False
    return True


def replaceINTO(tbl, name, data, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute("REPLACE INTO " + str(tbl) + " VALUES(?,DATETIME('now'),?)", (name, data))
    con.commit()
    if cur.rowcount == 0:
        return False
    return True

#def updateModel(tag, data, db_file):
def updateTable(tag, data, tbl, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute('REPLACE INTO '+str(tbl)+' VALUES(?,?)', (tag, data))
    rowid = cur.lastrowid
    con.commit()
    if cur.rowcount == 0:
        return None
    #return True
    return rowid

def updateTableTag(_id, tag, tbl, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute('UPDATE '+str(tbl)+' SET tag=? WHERE rowid=?', (tag, _id))
    con.commit()
    if cur.rowcount == 0:
        return False
    return True


def updateTraining(tag, data, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute("REPLACE INTO training VALUES(?,?)", (tag, data))
    con.commit()
    if cur.rowcount == 0:
        return False
    return True

def updateTrainingTag(_id, tag, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute("UPDATE training SET tag=? WHERE rowid=?", (tag, _id))
    con.commit()
    if cur.rowcount == 0:
        return False
    return True




# need to standardize on name, timestamp, data for all...
# create_fims  = "CREATE TABLE IF NOT EXISTS fims (name TEXT PRIMARY KEY NOT NULL,timestamp TEXT,data TEXT);"
#def updateTbl(tbl, name, data, db_file):
def updateData(tbl, name, data, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute("UPDATE " + str(tbl) + " SET data=?, timestamp=DATETIME('now') WHERE name=?", (data, name))
    con.commit()
    if cur.rowcount == 0:
        return False
    return True

def updateDataItem(item, val, tbl, name, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    #cur.execute("UPDATE " + str(tbl) + " SET data=?, timestamp=DATETIME('now') WHERE name=?", (data, name))
    cur.execute("UPDATE " + str(tbl) + " SET data=(select json_set(" + str(tbl) + ".data, '$." + str(item) + "', ? ) from " + str(tbl) + ") where name='" + str(name) + "';", (val,))
    con.commit()
    if cur.rowcount == 0:
        return False
    return True

def deleteDataItem(item, tbl, name, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute("UPDATE " + str(tbl) + " SET data=(select json_remove(" + str(tbl) + ".data, '$." + str(item) + "') from " + str(tbl) + ") where name=?", (name,))
    con.commit()
    if cur.rowcount == 0:
        return False
    return True


def updateFims(name, data, db_file): #not working?  ah, name,data...data,name
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute("UPDATE fims SET data=?, timestamp=DATETIME('now') WHERE name=?", (data, name))
    con.commit()
    if cur.rowcount == 0:
        return False
    return True


def deleteFromRowid(tbl, rowid, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute("DELETE FROM " + str(tbl) + " WHERE rowid=? ;", (rowid,))
    con.commit()
    if cur.rowcount == 0:
        return False
    return True

def deleteFrom(tbl, name, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute("DELETE FROM " + str(tbl) + " WHERE name=? ;", (name,))
    con.commit()
    if cur.rowcount == 0:
        return False
    return True

def deleteFromClmn(tbl, clmn, item, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute('DELETE FROM ' + str(tbl) + ' WHERE ' + str(clmn) + '=? ;', (item,))
    con.commit()
    if cur.rowcount == 0:
        return False
    return True

#def selectAllrowid(tbl, db_file):
#    con = sqlConnection(db_file)
#    cur = con.cursor()
#    cur.execute("SELECT rowid,* FROM " + str(tbl) + ";")
#    #cur.execute("SELECT * FROM " + str(tbl) + ";")
#    rows = cur.fetchall()
#    return rows

def selectAll(tbl, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute("SELECT * FROM " + str(tbl) + ";")
    rows = cur.fetchall()
    return rows
    #cur.execute("SELECT rowid,* FROM " + str(tbl) + ";")
    #if cur.rowcount == 0:
    #    return False

def getAll(tbl, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute('SELECT rowid,* FROM ' + str(tbl) + ';')
    rows = cur.fetchall()
    return rows

def getByID(tbl, rowid, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute("SELECT rowid,* FROM " + str(tbl) + " WHERE rowid=? ;", (rowid,))
    row = cur.fetchone()
    return row

def getByName(tbl, name, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute("SELECT * FROM " + str(tbl) + " WHERE name=? ;", (name,))
    row = cur.fetchone()
    return row

def getByOp(tbl, op, num, db_file):

# man bash
#       arg1 OP arg2
#              OP is one of -eq, -ne, -lt, -le, -gt, or -ge.  These  arithmetic
#              binary  operators return true if arg1 is equal to, not equal to,
#              less than, less than or equal to, greater than, or greater  than
#              or  equal  to arg2, respectively.  Arg1 and arg2 may be positive
#              or negative integers.

    con = sqlConnection(db_file)
    cur = con.cursor()

    #num = int(num)
    #https://www.sqlite.org/datatype3.html

    rows = None

    if '-eq' in op:
        cur.execute('SELECT * FROM '+str(tbl)+' WHERE tag = ? ;', (num,))

    elif '-gt' in op:
        cur.execute('SELECT * FROM '+str(tbl)+' WHERE tag > ? ;', (num,))

    elif '-lt' in op:
        cur.execute('SELECT * FROM '+str(tbl)+' WHERE tag < ? ;', (num,))

    elif '-ne' in op:
        cur.execute('SELECT * FROM '+str(tbl)+' WHERE tag <> ? ;', (num,))

    elif '-le' in op:
        cur.execute('SELECT * FROM '+str(tbl)+' WHERE tag <= ? ;', (num,))

    elif '-ge' in op:
        cur.execute('SELECT * FROM '+str(tbl)+' WHERE tag >= ? ;', (num,))

    #con = sqlConnection(db_file)
    #cur = con.cursor()
    #cur.execute('SELECT rowid,* FROM training WHERE tag=? ORDER by rowid DESC;', (tag,))

    rows = cur.fetchall()
    return rows

# HARDCODED table
def getAllTableTags(tag, tbl, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute('SELECT rowid,* FROM training WHERE tag=? ORDER by rowid DESC;', (tag,))
    rows = cur.fetchall()
    return rows


def getAllTrainingTags(tag, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute('SELECT rowid,* FROM training WHERE tag=? ORDER by rowid DESC;', (tag,))
    rows = cur.fetchall()
    return rows

def markAllTraining(tag, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute('UPDATE training SET tag=? ;', (tag,))
    con.commit()
    if cur.rowcount == 0:
        return False
    return True

#def markTrainingOn(name, db_file):
#    con = sqlConnection(db_file)
#    cur = con.cursor()
#    cur.execute('UPDATE training SET tag=? WHERE ;', (tag,))
#    con.commit()
#    if cur.rowcount == 0:
#        return False
#    return True



def getJob(name, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute('SELECT data FROM jobs WHERE name=? ;', (name,))
    row = cur.fetchone()
    #print(len(row))
    return row

def deleteJob(name, db_file):
    #print('let us delete...')
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute("DELETE FROM jobs WHERE name=? ;", (name,))
    con.commit()
    #print('cur.rowcount ' + str(cur.rowcount))
    if cur.rowcount == 0:
        return False
    return True

def clearAllJobs(db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute("DELETE FROM jobs;")
    cur.execute("REINDEX jobs;")
    con.commit()
    return True

def replaceCounts(name, count, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute("REPLACE INTO counts VALUES(?,?)", (name, count))
    con.commit()
    return True

#def replaceMemDBCounts(name, count, memdb):
#    try:
#        with memdb:
#            cur = memdb.cursor()
#            cur.execute("REPLACE INTO counts VALUES(?,?)", (name, count))
#            return True
#    except Exception as e:
#        print(str(e))
#        return False

#def getAllMemDBCounts(memdb):
#    try:
#        with memdb:
#            cur = con.cursor()
#            cur.execute("SELECT * FROM counts;")
#            rows = cur.fetchall()
#            return rows
#    except Exception as e:
#        print(str(e))
#        return None
    #Within one process it is possible to share an in-memory database if you use the file::memory:?cache=shared
    #but this is still not accessible from other another process.


def updateCounts(name, count, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    #sql = "UPDATE counts SET count='" + count + "' WHERE name='" + str(name) + "';" 
    #cur.execute(sql)
    #TypeError: can only concatenate str (not "int") to str
    cur.execute("UPDATE counts SET count=? WHERE name=? ;", (count, name))
    con.commit()
    return True

def getCount(name, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute('SELECT count FROM counts WHERE name=? ;', (name,))
    row = cur.fetchone()
    return row

def getAllCounts(db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute("SELECT * FROM counts;")
    rows = cur.fetchall()
    return rows

def insertINTOClientCommand(token, data, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    cur.execute("INSERT INTO client_commands VALUES(?,DATETIME('now'),?)", (token, data))
    con.commit()
    return True


#def updateData(tbl, name, data, db_file):
#def updateJobs(name, jdata, db_file):
#    con = sqlConnection(db_file)
#    cur = con.cursor()
#    sql = "UPDATE jobs SET data='" + jdata + "' WHERE name='" + str(name) + "';"
#    cur.execute(sql)
#    con.commit()
#    return True


def storeFile(_file, db_file):
    with open(_file, 'rb') as bfile:
        fblob = bfile.read()
    store_file = replaceINTO('files', _file, fblob, db_file)
    return store_file

def unstoreFile(_file, db_file):
    unstore_file = deleteFrom('files', _file, db_file)
    return unstore_file

def copyOccurrenceToTraining(name, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    tag='1'
    cur.execute('INSERT INTO training (tag,data) SELECT name,data FROM occurrence WHERE name=? ;', (name,))
    con.commit()
    if cur.rowcount == 0:
        return False
    return True


def copyOccurrenceToTable(name, db_file):
    con = sqlConnection(db_file)
    cur = con.cursor()
    tag='1'
    cur.execute('INSERT INTO training (tag,data) SELECT name,data FROM occurrence WHERE name=? ;', (name,))
    con.commit()
    if cur.rowcount == 0:
        return False
    return True






if __name__ == '__main__':
#uses built-in sqlite3
    pass




#!/usr/bin/env python3

__version__ = '20210405'

import libvirt
import sys
import json

class ListDomainsClass(object):
    def __init__(self, uri, host):
        self.uri = uri
        self.host = host

    def get(self):
        returnDict = {}
        uri_handle = str(self.uri) + str(self.host) + '/system'
        try:
            self.conn = libvirt.openReadOnly(uri_handle)
        except libvirt.libvirtError as e:
            return {"Error":str(e)}

        domains = self.conn.listAllDomains(0)
        if len(domains) != 0:
            for domain in domains:
                state = self.domstate(domain.name())
                returnDict[domain.name()] = str(state).lower()

        return returnDict

    def domstate(self,domName=None):
        dom = self.conn.lookupByName(domName)
        if dom == None:
            return str('NOTFOUND').lower()

        state, reason = dom.state()
        if state == libvirt.VIR_DOMAIN_NOSTATE:
            state='NOSTATE'
        elif state == libvirt.VIR_DOMAIN_RUNNING:
            state='RUNNING'
        elif state == libvirt.VIR_DOMAIN_BLOCKED:
            state='BLOCKED'
        elif state == libvirt.VIR_DOMAIN_PAUSED:
            state='PAUSED'
        elif state == libvirt.VIR_DOMAIN_SHUTDOWN:
            state='SHUTDOWN'
        elif state == libvirt.VIR_DOMAIN_SHUTOFF:
            state='SHUTOFF'
        elif state == libvirt.VIR_DOMAIN_CRASHED:
            state='CRASHED'
        elif state == libvirt.VIR_DOMAIN_PMSUSPENDED:
            state='PMSUSPENDED'
        else:
            state='UNKNOWN'

        return str(state).lower()

class ListDomainsDetailedClass(ListDomainsClass):
    def get(self):
        runningDict = {}
        uri_handle = str(self.uri) + str(self.host) + '/system'
        try:
            self.conn = libvirt.openReadOnly(uri_handle)
        except libvirt.libvirtError as e:
            return {"Error":str(e)}

        hvm_type = str(self.conn.getType())
        hypervisor_type = hvm_type.lower()

        nodeinfo = self.conn.getInfo()
        hvm_model = str(nodeinfo[0])
        hvm_mem = str(nodeinfo[1])
        hvm_cpu = str(nodeinfo[2])
        stats = self.conn.getCPUStats(libvirt.VIR_NODE_CPU_STATS_ALL_CPUS)

        hypervisor_cpu = {
          'count': str(hvm_cpu),
          'type': str(hvm_model),
          'kernel': str(stats['kernel']),
          'idle': str(stats['idle']),
          'user': str(stats['user']),
          'iowait': str(stats['iowait'])
        }

        memfree = self.conn.getFreeMemory()
        memfreeMB = memfree / 1024 / 1024
        memusedMB = int(hvm_mem) - int(memfreeMB)

        hypervisor_mem = {
          'total': str(hvm_mem),
          'used': str(memusedMB),
          'free': str(memfreeMB)
        }

        hypervisor = {
          'type': hypervisor_type,
          'cpu': hypervisor_cpu,
          'mem': hypervisor_mem
        }

        runningDict['hypervisor'] = hypervisor

        def domStatus(domName=None):
            runningStatus = {}

            dom = self.conn.lookupByName(domName)
            if dom == None:
                print(json.dumps({"Error":"Failed to find domain " + str(domName)}, indent=2))
                sys.exit(1)

            state = self.domstate(domName)

            state_lower_case = str(state).lower()
            runningStatus['state'] = state_lower_case

            active = dom.isActive()
            if active == True:
                ok=1
            else:
                return {'state': state}

            cpus = dom.maxVcpus() #dom cpu count
            if cpus != -1:
                pass
            else:
                print(json.dumps({"Error":"major error getting dom.Vcpus()"}, indent=2))
                sys.exit(1)

            cpu_stats = dom.getCPUStats(True)
            cpu_time = str(cpu_stats[0]['cpu_time'])
            system_time = str(cpu_stats[0]['system_time'])
            user_time = str(cpu_stats[0]['user_time'])

            dom_cpu = {
              'count': str(cpus),
              'cpu_time': cpu_time,
              'system_time': system_time,
              'user_time': user_time
            }

            runningStatus['cpu'] = dom_cpu

            mem = dom.maxMemory()
            if mem > 0:
                pass
            else:
                print(json.dumps({"Error":"major error getting dom.maxMemory()"}, indent=2))
                sys.exit(1)

            dom_mem = {}
            mem_stats  = dom.memoryStats()

            for name in mem_stats:
                if name:
                    dom_mem[name] = str(mem_stats[name])

            runningStatus['mem'] = dom_mem

            from xml.dom import minidom
            raw_xml = dom.XMLDesc(0)
            xml = minidom.parseString(raw_xml)

            diskTypes = xml.getElementsByTagName('disk')
            diskDict = {}
            source = None
            for diskType in diskTypes:
                diskNodes = diskType.childNodes
                for diskNode in diskNodes:
                    if diskNode.nodeName[0:1] != '#':
                        if diskNode.nodeName == 'source':
                            for attr in diskNode.attributes.keys():
                                if diskNode.attributes[attr].name == 'dev':
                                    source = str(diskNode.attributes[attr].value)
                                elif diskNode.attributes[attr].name == 'file':
                                    source = str(diskNode.attributes[attr].value)
                        if diskNode.nodeName == 'target':
                            for attr in diskNode.attributes.keys():
                                if diskNode.attributes[attr].name == 'dev':
                                    target = str(diskNode.attributes[attr].value)
                                    if source is not None:
                                        rd_req, rd_bytes, wr_req, wr_bytes, err = dom.blockStats(source)
                                    else:
                                        rd_req, rd_bytes, wr_req, wr_bytes, err = None

                                    diskDict[target] = {
                                      'device': str(source),
                                      'read_requests_issued': str(rd_req),
                                      'bytes_read': str(rd_bytes),
                                      'write_requests_issued': str(wr_req),
                                      'bytes_written': str(wr_bytes),
                                      'number_of_errors': str(err)
                                    }
            runningStatus['disk'] = diskDict


            netDict = {}
            interfaceTypes = xml.getElementsByTagName('interface')
            count = 0
            for interfaceType in interfaceTypes:
                netname = 'eth' + str(count)
                count = count + 1
                ifaceDict = {}
                interfaceNodes = interfaceType.childNodes
                for interfaceNode in interfaceNodes:
                    if interfaceNode.nodeName[0:1] != '#':
                        for attr in interfaceNode.attributes.keys():
                            if interfaceNode.attributes[attr].name == 'address':
                                mac = str(interfaceNode.attributes[attr].value)
                                ifaceDict['mac'] = mac

                            if interfaceNode.attributes[attr].name == 'bridge':
                                bridge = str(interfaceNode.attributes[attr].value)
                                ifaceDict['bridge'] = bridge

                            if interfaceNode.attributes[attr].name == 'dev':
                                iface = str(interfaceNode.attributes[attr].value)

                                stats = dom.interfaceStats(iface)

                                ifaceDict['read_bytes'] = str(stats[0])
                                ifaceDict['read_packets'] = str(stats[1])
                                ifaceDict['read_errors'] = str(stats[2])
                                ifaceDict['read_drops'] = str(stats[3])
                                ifaceDict['write_bytes'] = str(stats[4])
                                ifaceDict['write_packets'] = str(stats[5])
                                ifaceDict['write_errors'] = str(stats[6])
                                ifaceDict['write_drops'] = str(stats[7])
                    netDict[netname] = ifaceDict
            runningStatus['net'] = netDict

            return runningStatus

        domDict = {}
        domains = self.conn.listAllDomains(0)
        if len(domains) != 0:
            for domain in domains:
                status = domStatus(domain.name())
                domDict[domain.name()] = status

        runningDict['domains'] = domDict

        self.conn.close()
        return runningDict

if __name__ == "__main__":

    #uri = 'qemu+tcp://'
    uri = 'qemu://'
    host = ''
    runningDict = ListDomainsDetailedClass(uri, host).get()
    print(json.dumps(runningDict, indent=2, sort_keys=True))






#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__version__ = "1.8.4"

import sys

if sys.version_info < (3, 8, 1):
    raise RuntimeError('Requires Python version 3.8.1 or higher. This version: ' + str(sys.version_info))

if sys.platform not in ('linux','linux2','darwin','cygwin'):
    raise RuntimeError('Platform not supported. This platform: ' + str(sys.platform))

import sqlite3

if sqlite3.sqlite_version_info < (3, 28, 0):
    raise RuntimeError('Requires Python sqlite3 library 3.28.0 or higher. This version: ' + str(sqlite3.sqlite_version))

import os
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__),'..')))

import json

from tools import *
from store import *


def usage():
    print(sys.argv[0] + ''' [option]

    options:

        list-proms

        list-configs
          update-config name data
          delete-config name
          clear-configs

        nmap-net net
        ping-net ip/net
        net-scan net

        port-scan [ip/net] [level]
          list-nmaps
          nmap ip [level]
          del-nmap ip
          clear-nmaps

        vuln-scan [ip/net]
          list-vulns [id]
          del-vuln id
          clear-vulns
          check-vuln id
          email-vuln id

        arps
        manuf mac
        lsof port
        rdns ip [srv]
        myip

        ip-whois ip

        udp ip port
        udpscan ip port
        tcp ip port

        list-macs
          update-manuf mac
          update-dns mac ip

        listening
          listening-detailed
          listening-details port
          listening-allowed
          listening-alerts
          listening-allow port
          listening-remove port

        established
          established-lsof
          established-rules
          established-rules-filter
          established-rule ALLOW|DENY proto laddr lport faddr fport
          established-alerts
          delete-established-rule rowid
          clear-established-rules

        list-ips
          update-ip ip data
          update-ip-item ip item value
          delete-ip-item ip item value
          del-ip ip
          clear-ips

        list-jobs
          list-jobs-available
          update-job name data
          delete-job name
          clear-jobs

        list-rules
          update-rule name data
          delete-rule name
          clear-rules

        list-reports
          update-report name data
          delete-report name
          clear-reports

        list-alerts
          delete-alert id
          run-alert name
          update-alert name data
          run-alert name
          clear-alerts

        list-fims
          list-fims-changed
          check-fim [name]
          b2sum-fim [name]
          b2sum /dir/file
          update-fim name data
          delete-fim id
        add-fim name /dir/file
        del-fim name /dir/file

        list-files
          add-file /dir/file
          del-file /dir/file
        fim-restore /dir/file [/dir/file]
        fim-diff
        clear-files

        file-type /dir/file

        list-avs
        av-scan dir|file

        list-proms-db
          update-prom-db name data
          clear-proms-db

        list-b2sums
          clear-b2sums

        list-counts
          clear-counts

        register-client job_name server_key
        remote-client job_name

        list-api-tokens
          update-api-token token data
          delete-api-token token
          clear-api-tokens

        list-client-commands
          clear-client-commands
          delete-client-command rowid

        base64 <string>
        unbase64 <base64>

        list-model [id|tags tag]
          update-model tag json
          update-model-tag id tag
          delete-model id
          clear-model

        list-training [id|tags tag]
          update-training tag json
          update-training-tag id tag
          delete-training id
          clear-training

        list-occurrence [name|-eq,-gt,-lt,-ne,-le,-ge num]
          delete-occurrence name
          clear-occurrence

        copy-occurrence name

        sample-logstream count
        # ? mark-training tag
        # ? mark-training-on name

        list-system-profile
          list-system-profile-full
          gen-system-profile
          get-system-profile-name name
          get-system-profile-rowid rowid
          del-system-profile-name name
          del-system-profile-rowid rowid
          clear-system-profile

        diff-system-profile-rowid rowid rowid
        get-system-profile-data rowid data

        tail file
        logstream
        logstream-json
        logstream-keys
        run-create-db
        run-ps

        ---

        sentry [debug|info|warning|error|critical]


        config

                logstream:
                    rules
                    sklearn naive_bayes.MultinomialNB
                            naive_bayes.BernoulliNB
                            neural_network.MLPClassifier
                tail:
                    rules

                api_server
                http_server
                pushgateway


        list-keys
        list-keys-metric
        list-vals
        get-key key
        expire-keys key1 key2 key3...

Version: {} '''.format(__version__))


def printArps():
    arpTbl = tools.getArps()
    for k,v in arpTbl.items():
        if (v == '(incomplete)') or (v == '<incomplete>'):
            continue
        print(v,k)
    return True


def main():

    db_store = 'sentinel.db'
    db_manuf = str(os.path.dirname(__file__)) + '/db/manuf'

    if sys.argv[1:]:

        if sys.argv[1] == '--version':
            print(__version__)
            sys.exit(0)

        if sys.argv[1] == 'base64':
            string = sys.argv[2]
            base64_output = base64.b64encode(string.encode('utf-8')).decode('utf-8')
            print(base64_output)
            sys.exit(0)

        if sys.argv[1] == 'unbase64':
            string = sys.argv[2]
            unbase64_output = base64.b64decode(string.encode('utf-8')).decode('utf-8')
            print(unbase64_output)
            sys.exit(0)

        if sys.argv[1] == 'manuf':
            mac = sys.argv[2]
            mfname = store.get_manuf(mac, db_manuf)
            print(mfname)
            sys.exit(0)

        if sys.argv[1] == 'arps':
            printArps()
            sys.exit(0)

        if sys.argv[1] == 'list-macs':
            store.print_all(db_store)
            sys.exit(0)

        if sys.argv[1] == 'update-manuf':
            mac = sys.argv[2]
            mfname = store.get_manuf(mac, db_manuf)
            update = store.update_data_manuf(mac, mfname, db_store)
            print(update)
            sys.exit(0)

        if sys.argv[1] == 'rdns':
            ip = sys.argv[2]
            try: srv = sys.argv[3]
            except IndexError: srv = None
            dnsname = tools.getNSlookup(ip, srv)
            print(dnsname)
            sys.exit(0)

        if sys.argv[1] == 'update-dns':
            mac = sys.argv[2]
            ip = sys.argv[3]
            #dnsname = tools.getDNSName(ip)
            #update = store.update_data_dns(mac, dnsname, db_store)
            import threading
            dns = store.DNSUpDateTask()
            t = threading.Thread(target=dns.run, args=(mac,ip,db_store,))
            t.start()
            #print(t)
            sys.exit(0)

        if sys.argv[1] == 'ping-net':
            ip = sys.argv[2]
            pn = tools.pingNet(ip)
            print(pn)
            sys.exit(0)

        if sys.argv[1] == 'nmap-net':
            ip = sys.argv[2]
            pn = tools.nmapNet(ip)
            print(pn)
            sys.exit(0)

        if sys.argv[1] == 'net-scan':
            net = sys.argv[2]
            scan = tools.netScan(net, db_store, {}, 'net-scan')
            print(scan)
            sys.exit(0)


        if sys.argv[1] == 'listening':
            p = tools.printListenPorts()
            sys.exit(0)

        if sys.argv[1] == 'listening-detailed':
            p = tools.printListenPortsDetailed()
            sys.exit(0)

        if sys.argv[1] == 'listening-details':
            port = sys.argv[2]
            p = tools.printLsOfPort(port)
            sys.exit(0)

        if sys.argv[1] == 'listening-allowed':
            p = store.printListeningAllowed(db_store)
            sys.exit(0)

        if sys.argv[1] == 'listening-allow':
            port = sys.argv[2]
            insert = store.insertAllowedPort(port, db_store)
            print(insert)
            sys.exit(0)

        if sys.argv[1] == 'listening-remove':
            port = sys.argv[2]
            remove = store.deleteAllowedPort(port, db_store)
            print(remove)
            sys.exit(0)

        if sys.argv[1] == 'listening-alerts':
            alerts = store.printListeningAlerts(db_store)
            sys.exit(0)

        if sys.argv[1] == 'established':
            established = tools.printEstablished()
            sys.exit(0)

        if sys.argv[1] == 'established-lsof':
            established = tools.printEstablishedLsOf()
            sys.exit(0)

        if sys.argv[1] == 'established-rules':
            established_rules = tools.printEstablishedRules(db_store)
            sys.exit(0)

        if sys.argv[1] == 'established-rule':
            #established-rule proto laddr lport faddr fport
            rule  = sys.argv[2]
            proto = sys.argv[3]
            laddr = sys.argv[4]
            lport = sys.argv[5]
            faddr = sys.argv[6]
            fport = sys.argv[7]
            insert_rule = store.insertEstablishedRules(rule, proto, laddr, lport, faddr, fport, db_store)
            sys.exit(0)

        if sys.argv[1] == 'established-rules-filter':
            print_alerts = tools.printEstablishedRulesMatch(db_store)
            sys.exit(0)

        if sys.argv[1] == 'established-alerts':
            print_alerts = tools.printEstablishedAlerts(db_store)
            sys.exit(0)

        if sys.argv[1] == 'delete-established-rule':
            rowid = sys.argv[2]
            delete = store.deleteFromRowid('established', rowid, db_store)
            print(delete)
            sys.exit(0)

        if sys.argv[1] == 'clear-established-rules':
            clear = store.clearAll('established', db_store)
            print(clear)
            sys.exit(0)

        if sys.argv[1] == 'clear-configs':
            clear = store.clearAll('configs', db_store)
            print(clear)
            sys.exit(0)

        if sys.argv[1] == 'lsof':
            port = sys.argv[2]
            lsof = tools.printLsOfPort(port)
            sys.exit(0)

        if sys.argv[1] == 'nmap':
            ip = sys.argv[2]
            try: level = sys.argv[3]
            except IndexError: level = 1
            scan = tools.nmapScan(ip, level)
            update = store.replaceNmaps(ip, scan, db_store)
            print(str(update) + ' ' + str(scan))
            sys.exit(0)

        if sys.argv[1] == 'list-ips':
            rows = store.selectAll('ips', db_store)
            for row in rows:
                print(row)
            sys.exit(0)

        if sys.argv[1] == 'add-ip':
            ip = sys.argv[2]
            insert = store.insertIPs(ip, db_store)
            print(insert)
            sys.exit(0)

        if sys.argv[1] == 'del-ip':
            ip = sys.argv[2]
            _del = store.deleteIPs(ip, db_store)
            print(_del)
            sys.exit(0)

        if sys.argv[1] == 'update-ip':
            ip = sys.argv[2]
            data = sys.argv[3]

            try: valid_json = json.loads(data)
            except json.decoder.JSONDecodeError:
                print('invalid json')
                sys.exit(1)
            replace = store.replaceINTO('ips', ip, data, db_store)
            print(replace)
            sys.exit(0)

        if sys.argv[1] == 'update-ip-item':
            name = sys.argv[2]
            item = sys.argv[3]
            val  = sys.argv[4]
            update = store.updateDataItem(item, val, 'ips', name, db_store)
            print(update)
            sys.exit(0)

        if sys.argv[1] == 'delete-ip-item':
            name = sys.argv[2]
            item = sys.argv[3]
            delete = store.deleteDataItem(item, 'ips', name, db_store)
            print(delete)
            sys.exit(0)

        if sys.argv[1] == 'clear-ips':
            clear = store.clearAll('ips', db_store)
            print(clear)
            sys.exit(0)

        if sys.argv[1] == 'discover-net':
            ipnet = None
            level = None
            try:
                ipnet = sys.argv[2]
                level = sys.argv[3]
            except IndexError: pass

            if ipnet is None:
                ipnet = tools.getIfconfigIPv4()
            else:
                i = ipnet.split('.')
                if len(i) == 1:
                    level = sys.argv[2]
                    ipnet = tools.getIfconfigIPv4()
            if level is None:
                level = 1

            run_discovery = tools.runDiscoverNet(ipnet, level, db_store)
            print(run_discovery)
            sys.exit(0)

        if sys.argv[1] == 'list-nmaps':
            scans = store.getNmaps(db_store)
            for row in scans:
                print(row)
            sys.exit(0)

        if sys.argv[1] == 'del-nmap':
            ip = sys.argv[2]
            del_ = store.deleteNmaps(ip, db_store)
            print(del_)
            sys.exit(0)

        if sys.argv[1] == 'clear-nmaps':
            clear = store.clearAllNmaps(db_store)
            print(clear)
            sys.exit(0)

        if sys.argv[1] == 'list-vulns':
            try: vid = sys.argv[2]
            except IndexError: vid = None
            run = tools.printVulnScan(db_store, vid)
            sys.exit(0)

        if sys.argv[1] == 'del-vuln':
            ip = sys.argv[2]
            del_ = store.deleteVulns(ip, db_store)
            print(del_)
            sys.exit(0)

        if sys.argv[1] == 'clear-vulns':
            clear = store.clearAllVulns(db_store)
            print(clear)
            sys.exit(0)

        if sys.argv[1] == 'check-vuln':
            vid = sys.argv[2]
            data = store.getVulnData(vid, db_store)
            run = tools.processVulnData(data)
            print(run)
            sys.exit(0)

        if sys.argv[1] == 'email-vuln':
            vid = sys.argv[2]
            data = store.getVulnData(vid, db_store)
            subject = 'sentinel vuln-scan'
            email = tools.sendEmail(subject, data, db_store)
            print(email)
            sys.exit(0)

        if sys.argv[1] == 'myip':
            myip = tools.getIfconfigIPv4()
            print(myip)
            sys.exit(0)

        if sys.argv[1] == 'udp':
            ip = sys.argv[2]
            port = sys.argv[3]
            run = tools.nmapUDP(ip, port)
            print(run)
            sys.exit(0)

        if sys.argv[1] == 'udpscan':
            ip = port = None
            try:
                ip = sys.argv[2]
                port = sys.argv[3]
            except IndexError: pass
            run = tools.nmapUDPscan(ip, port)
            print(run)
            sys.exit(0)

        if sys.argv[1] == 'tcp':
            ip = sys.argv[2]
            port = sys.argv[3]
            run = tools.nmapTCP(ip, port)
            print(run)
            sys.exit(0)

        if sys.argv[1] == 'list-detects':
            try: id_ = sys.argv[2]
            except IndexError: id_ = None
            run = tools.printDetectScan(db_store, id_)
            sys.exit(0)

        if sys.argv[1] == 'detect-scan':
            ip = sys.argv[2]
            scan = tools.nmapDetectScanStore(ip, db_store)
            print(str(scan))
            sys.exit(0)

        if sys.argv[1] == 'del-detect':
            id_ = sys.argv[2]
            del_ = store.deleteDetect(id_, db_store)
            print(del_)
            sys.exit(0)

        if sys.argv[1] == 'clear-detects':
            clear = store.clearAllDetects(db_store)
            print(clear)
            sys.exit(0)

        if sys.argv[1] == 'port-scan':
            ipnet = None
            level = 1
            try:
                ipnet = sys.argv[2]
                level = sys.argv[3]
            except IndexError: pass

            if ipnet is None:
                myip = tools.getIfconfigIPv4()
                ipn = tools.getIpNet(myip)
                print('discover net: ' + str(ipn))
                ipnet = tools.nmapNet(ipn)
            else:
                i = ipnet.split('.')

                if len(i) == 1:
                    level = sys.argv[2]
                    myip = tools.getIfconfigIPv4()
                    ipn = tools.getIpNet(myip)
                    print('discover net: ' + str(ipn))
                    ipnet = tools.nmapNet(ipn)
                else:
                    if tools.isNet(ipnet):
                        print('discover net: ' + str(ipnet))
                        ipnet = tools.nmapNet(ipnet)
            
            if type(ipnet) == str:
                ipnet = ipnet.split()
            scan = tools.runNmapScanMultiProcess(ipnet, level, db_store)
            print(scan)
            sys.exit(0)

        if sys.argv[1] == 'vuln-scan':
            try: ipnet = sys.argv[2]
            except IndexError: ipnet = None

            if ipnet is None:
                myip = tools.getIfconfigIPv4()
                ipn = tools.getIpNet(myip)
                print('discover net: ' + str(ipn))
                ipnet = tools.nmapNet(ipn)
            else:
                if tools.isNet(ipnet):
                    print('discover net: ' + str(ipnet))
                    ipnet = tools.nmapNet(ipnet)

            if type(ipnet) == str:
                ipnet = ipnet.split()

            scan = tools.runNmapVulnMultiProcess(ipnet, db_store)
            print(scan)
            sys.exit(0)

        if sys.argv[1] == 'detect-scan-net':
            ipnet = None
            try: ipnet = sys.argv[2]
            except IndexError: pass
            if ipnet is None:
                ipnet = tools.getIfconfigIPv4()
            ipn = tools.getIpNet(ipnet)
            print('ipnet: ' + ipn)
            hostLst = tools.nmapNet(ipn)
            scan = tools.runNmapDetectMultiProcess(hostLst, db_store)
            print(scan)
            sys.exit(0)

        if sys.argv[1] == 'list-configs':
            run = tools.printConfigs(db_store)
            print(run)
            sys.exit(0)

        if sys.argv[1] == 'update-config':
            name = sys.argv[2]
            data = sys.argv[3]
            try: valid_json = json.loads(data)
            except json.decoder.JSONDecodeError: 
                print('invalid json') 
                sys.exit(1)
            run = store.replaceINTO('configs', name, data, db_store)
            print(run)
            sys.exit(0)

        if sys.argv[1] == 'delete-config':
            rowid = sys.argv[2]
            run = store.deleteFrom('configs', rowid, db_store)
            print(run)
            sys.exit(0)

        if sys.argv[1] == 'list-rules':
            rows = store.selectAll('rules', db_store)
            for row in rows:
                print(row)
            sys.exit(0)

        if sys.argv[1] == 'update-rule':
            name = sys.argv[2]
            data = sys.argv[3]
            try: valid_json = json.loads(data)
            except json.decoder.JSONDecodeError:
                print('invalid json')
                sys.exit(1)
            run = store.replaceINTO('rules', name, data, db_store)
            print(run)
            sys.exit(0)

        if sys.argv[1] == 'delete-rule':
            name = sys.argv[2]
            run = store.deleteFrom('rules', name, db_store)
            print(run)
            sys.exit(0)

        if sys.argv[1] == 'clear-rules':
            clear = store.clearAll('rules', db_store)
            print(clear)
            sys.exit(0)

        if sys.argv[1] == 'list-jobs':
            rows = store.selectAll('jobs', db_store)
            for row in rows:
                print(row)
            sys.exit(0)

        if sys.argv[1] == 'update-job':
            name = sys.argv[2]
            data = sys.argv[3]
            try: valid_json = json.loads(data)
            except json.decoder.JSONDecodeError: 
                print('invalid json') 
                sys.exit(1)
            run = store.replaceINTO('jobs', name, data, db_store)
            print(run)
            sys.exit(0)

        if sys.argv[1] == 'delete-job':
            name = sys.argv[2]
            run = store.deleteJob(name, db_store)
            print(run)
            sys.exit(0)

        if sys.argv[1] == 'clear-jobs':
            clear = store.clearAllJobs(db_store)
            print(clear)
            sys.exit(0)

#
# run-job name
#        if sys.argv[1] == 'run-job':
#            name = sys.argv[2]
#            run = tools.runJob(name, db_store)
#            print(str(run))
#            sys.exit(0)
#def runJob(name, db_store, gDict):

        if sys.argv[1] == 'sentry':
            try: v = sys.argv[2]
            except IndexError: v = False
            run = tools.sentryMode(db_store, verbose=v)
            print(str(run))
            sys.exit(0)

#
# list-jobs-running

        #if sys.argv[1] == 'list-jobs-running':
        #    run = tools.listRunning(db_store)
        #    sys.exit(0)

#def listRunning(db_store):
#    rows = store.getAllCounts(db_store)
#    for row in rows:
#        print(row)
#    return True

#def getAllCounts(db_file):
#    con = sql_connection(db_file)
#    cur = con.cursor()
#    cur.execute("SELECT * FROM counts;")
#    rows = cur.fetchall()
#    return rows



        if sys.argv[1] == 'b2sum':
            _file = sys.argv[2]
            b2sum = tools.b2sum(_file)
            print(_file + ' ' + b2sum)
            sys.exit(0)

        if sys.argv[1] == 'b2sum-fim':
            try: name = sys.argv[2]
            except IndexError: name = None
            if name is None:
                fims = store.selectAll('fims', db_store)
                for i in fims:
                    name = i[0]
                    run = tools.b2sumFim(name, db_store)
                    print(str(name) + ' ' + str(run))
            else:
                run = tools.b2sumFim(name, db_store)
                print(str(run))
            sys.exit(0)

        if sys.argv[1] == 'check-fim':
            try: name = sys.argv[2]
            except IndexError: name = None
            if name is None:
                fims = store.selectAll('fims', db_store)
                for i in fims:
                    name = i[0]
                    run = tools.printFim(name, db_store)
                    print(str(name) + ' ' + str(run))
            else:
                run = tools.printFim(name, db_store)
                print(str(run))
            sys.exit(0)

        if sys.argv[1] == 'list-fims':
            run = tools.printAllFims(db_store)
            print(run)
            sys.exit(0)

        if sys.argv[1] == 'list-fims-changed':
            run = tools.printAllFimsChanged(db_store)
            print(run)
            sys.exit(0)

        if sys.argv[1] == 'add-fim':
            name = sys.argv[2]
            _file = sys.argv[3]
            add = tools.addFimFile(name, _file, db_store)
            print(str(add))
            sys.exit(0)

        if sys.argv[1] == 'del-fim':
            name = sys.argv[2]
            _file = sys.argv[3]
            add = tools.delFimFile(name, _file, db_store)
            print(str(add))
            sys.exit(0)

        if sys.argv[1] == 'update-fim':
            name = sys.argv[2]
            data = sys.argv[3]
            try: valid_json = json.loads(data)
            except json.decoder.JSONDecodeError:
                print('invalid json')
                sys.exit(1)
            run = store.replaceINTO('fims', name, data, db_store)
            print(run)
            sys.exit(0)

        if sys.argv[1] == 'list-reports':
            reports = store.selectAll('reports', db_store)
            for row in reports:
                print(row)
            sys.exit(0)

        if sys.argv[1] == 'delete-report':
            name = sys.argv[2]
            delete = store.deleteFrom('reports', name, db_store)
            print(delete)
            sys.exit(0)

        if sys.argv[1] == 'clear-reports':
            clear = store.clearAll('reports', db_store)
            print(clear)
            sys.exit(0)

        if sys.argv[1] == 'update-report':
            name = sys.argv[2]
            data = sys.argv[3]
            try: valid_json = json.loads(data)
            except json.decoder.JSONDecodeError:
                print('invalid json')
                sys.exit(1)
            run = store.replaceINTO('reports', name, data, db_store)
            print(run)
            sys.exit(0)

        if sys.argv[1] == 'list-alerts':
            alerts = store.selectAll('alerts', db_store)
            for row in alerts:
                print(row)
            sys.exit(0)

        if sys.argv[1] == 'delete-alert':
            name = sys.argv[2]
            delete = store.deleteFrom('alerts', name, db_store)
            print(delete)
            sys.exit(0)

        if sys.argv[1] == 'clear-alerts':
            clear = store.clearAll('alerts', db_store)
            print(clear)
            sys.exit(0)

        if sys.argv[1] == 'update-alert':
            name = sys.argv[2]
            data = sys.argv[3]
            try: valid_json = json.loads(data)
            except json.decoder.JSONDecodeError:
                print('invalid json')
                sys.exit(1)
            run = store.replaceINTO('alerts', name, data, db_store)
            print(run)
            sys.exit(0)

        if sys.argv[1] == 'run-alert':
            name = sys.argv[2]
            run = tools.runAlert(name, db_store)
            print(str(run))
            sys.exit(0)

        if sys.argv[1] == 'run-create-db':
            run = store.createDB(db_store)
            print(str(run))
            sys.exit(0)

        if sys.argv[1] == 'ntp-check':
            run = tools.ntpCheck('ntp-check-cli', db_store, {}, 'ntp-check-cli', verbose=True)
            print(run)
            sys.exit(0)

        if sys.argv[1] == 'run-ps':
            #import modules.ps.ps
            #run = modules.ps.ps.get_ps()
            #from .modules.ps import ps
            from modules.ps import ps
            run = ps.get_ps()
            print(run)
            sys.exit(0)

        if sys.argv[1] == 'list-jobs-available':
            for k,v in tools.options.items():
                print(k)
            sys.exit(0)

        if sys.argv[1] == 'list-counts':
            reports = store.selectAll('counts', db_store)
            for row in reports:
                print(row)
            sys.exit(0)

        if sys.argv[1] == 'clear-counts':
            clear = store.clearAll('counts', db_store)
            print(clear)
            sys.exit(0)

        if sys.argv[1] == 'list-client-commands':
            #reports = store.selectAll('client_commands', db_store)
            reports = store.getAll('client_commands', db_store)
            for row in reports:
                print(row)
            sys.exit(0)

        if sys.argv[1] == 'delete-client-command':
            rowid = sys.argv[2]
            delete = store.deleteFromRowid('client_commands', rowid, db_store)
            print(delete)
            sys.exit(0)

        if sys.argv[1] == 'clear-client-commands':
            clear = store.clearAll('client_commands', db_store)
            print(clear)
            sys.exit(0)

        if sys.argv[1] == 'remote-client':
            job_name = sys.argv[2]
            run = tools.RClient(job_name, db_store)
            print(run)
            sys.exit(0)

        if sys.argv[1] == 'register-client':
            job_name = sys.argv[2]
            server_key = sys.argv[3]

            run = tools.register_client(job_name, server_key, db_store)
            print(run)

            sys.exit(0)

        if sys.argv[1] == 'list-api-tokens':
            reports = store.selectAll('bearer', db_store)
            for row in reports:
                print(row)
            sys.exit(0)

        if sys.argv[1] == 'clear-api-tokens':
            clear = store.clearAll('bearer', db_store)
            print(clear)
            sys.exit(0)

        if sys.argv[1] == 'update-api-token':
            token = sys.argv[2]
            data = sys.argv[3]
            try: valid_json = json.loads(data)
            except json.decoder.JSONDecodeError:
                print('invalid json')
                sys.exit(1)
            run = store.replaceINTO('bearer', token, data, db_store)
            print(run)
            sys.exit(0)

        if sys.argv[1] == 'delete-api-token':
            token = sys.argv[2]
            delete = store.deleteFromClmn('bearer', 'token', token, db_store)
            print(delete)
            sys.exit(0)


        if sys.argv[1] == 'list-proms':
            _prom = str(db_store) + '.prom'
            with open(_prom, 'r') as _file:
                lines = _file.readlines()
                for line in lines:
                    print(line.strip('\n'))
            sys.exit(0)

        if sys.argv[1] == 'list-proms-db':
            proms = store.selectAll('proms', db_store)
            for row in proms:
                print(row)
            sys.exit(0)

        if sys.argv[1] == 'clear-proms-db':
            clear = store.clearAll('proms', db_store)
            print(clear)
            sys.exit(0)

        if sys.argv[1] == 'update-prom-db':
            name = sys.argv[2]
            data = sys.argv[3]
            run = store.replaceINTOproms(name, data, db_store)
            print(run)
            sys.exit(0)

        if sys.argv[1] == 'file-type':
            #import modules.gitegridy.gitegridy as git
            #from .modules.gitegridy import gitegridy as git
            from modules.gitegridy import gitegridy as git
            _file = sys.argv[2]
            file_type = git.fileType(_file)
            print(file_type)
            sys.exit(0)

        if sys.argv[1] == 'add-file':
            _file = sys.argv[2]
            store_file = store.storeFile(_file, db_store)
            print(store_file)
            sys.exit(0)

        if sys.argv[1] == 'del-file':
            _file = sys.argv[2]
            unstore_file = store.unstoreFile(_file, db_store)
            print(unstore_file)
            sys.exit(0)

        if sys.argv[1] == 'list-files':
            list_files = store.selectAll('files', db_store)
            for row in list_files:
                print(row[0], row[1])
            sys.exit(0)

        if sys.argv[1] == 'clear-files':
            clear = store.clearAll('files', db_store)
            print(clear)
            sys.exit(0)

        if sys.argv[1] == 'fim-diff':
            _file = sys.argv[2]
            fim_diff = tools.fimDiff(_file, db_store)
            print(fim_diff)
            sys.exit(0)

        if sys.argv[1] == 'fim-restore':
            _file = sys.argv[2]
            try: _dest = sys.argv[3]
            except IndexError: _dest = None

            store_file_ = store.getData('files', _file, db_store)
            store_file_blob = store_file_[0]
            
            if _dest:
                dest = _dest
            else:
                dest = _file

            with open(dest, 'wb+') as outfile:
                outfile.write(store_file_blob)

            print('fim-restore ' + dest)
            sys.exit(0)

        if sys.argv[1] == 'av-scan':
            filedir = sys.argv[2]
            av_scan = tools.avScan(filedir, db_store)
            print(av_scan)
            sys.exit(0)

        if sys.argv[1] == 'tail':
            _file = sys.argv[2]
            for line in tools.tail(_file):
                print(line)
            sys.exit(0)

        if sys.argv[1] == 'logstream':
            for line in tools.logstream():
                print(line)
            sys.exit(0)

        if sys.argv[1] == 'logstream-json':
            for line in tools.logstream():
                print(line.decode('utf-8'))
            sys.exit(0)

        if sys.argv[1] == 'logstream-keys':
            for line in tools.logstream():
                jline = json.loads(line.decode('utf-8'))
                n = len(jline.keys())
                print(n, ' ' , jline.keys())
            sys.exit(0)

        if sys.argv[1] == 'list-b2sums':
            rows = store.selectAll('b2sum', db_store)
            for row in rows:
                print(row)
            sys.exit(0)

        if sys.argv[1] == 'clear-b2sums':
            clear = store.clearAll('b2sum', db_store)
            print(clear)
            sys.exit(0)

        #if sys.argv[1] == 'list-sshwatch':
        #    rows = store.selectAll('sshwatch', db_store)
        #    for row in rows:
        #        print(row)
        #    sys.exit(0)

        #if sys.argv[1] == 'clear-sshwatch':
        #    clear = store.clearAll('sshwatch', db_store)
        #    print(clear)
        #    sys.exit(0)

        if sys.argv[1] == 'clear-training':
            clear = store.clearAll('training', db_store)
            print(clear)
            sys.exit(0)

        if sys.argv[1] == 'clear-model':
            clear = store.clearAll('model', db_store)
            print(clear)
            sys.exit(0)

        if sys.argv[1] == 'list-training':
            try: _id = sys.argv[2]
            except IndexError: _id = None

            if _id:

                if _id  == 'tags':
                    _tag = sys.argv[3]
                    rows = store.getAllTrainingTags(_tag, db_store)
                    for row in rows:
                        print(row)
                else:
                    row = store.getByID('training', _id, db_store)
                    print(row)

            else:
                rows = store.getAll('training', db_store)
                for row in rows:
                    print(row)
            sys.exit(0)

        if sys.argv[1] == 'list-model':
            try: _id = sys.argv[2]
            except IndexError: _id = None

            if _id:

                if _id  == 'tags':
                    _tag = sys.argv[3]
                    #rows = store.getAllTrainingTags(_tag, db_store)
                    rows = store.getAllTableTags(_tag, 'model', db_store)
                    for row in rows:
                        print(row)
                else:
                    row = store.getByID('model', _id, db_store)
                    print(row)

            else:
                rows = store.getAll('model', db_store)
                for row in rows:
                    print(row)
            sys.exit(0)


        if sys.argv[1] == 'update-model':
            tag = sys.argv[2]
            data = sys.argv[3]
            try: valid_json = json.loads(data)
            except json.decoder.JSONDecodeError:
                print('invalid json')
                sys.exit(1)
            #run = store.updateTraining(tag, data, db_store)
            run = store.updateTable(tag, data, 'model', db_store)
            print(run)
            sys.exit(0)

        if sys.argv[1] == 'update-model-tag':
            _id = sys.argv[2]
            tag = sys.argv[3]
            #run = store.updateTrainingTag(_id, tag, db_store)
            run = store.updateTableTag(_id, tag, 'model', db_store)
            print(run)
            sys.exit(0)


        if sys.argv[1] == 'update-training':
            tag = sys.argv[2]
            data = sys.argv[3]
            try: valid_json = json.loads(data)
            except json.decoder.JSONDecodeError:
                print('invalid json')
                sys.exit(1)
            run = store.updateTraining(tag, data, db_store)
            print(run)
            sys.exit(0)

        if sys.argv[1] == 'update-training-tag':
            _id = sys.argv[2]
            tag = sys.argv[3]
            run = store.updateTrainingTag(_id, tag, db_store)
            print(run)
            sys.exit(0)

        if sys.argv[1] == 'delete-training':
            rowid = sys.argv[2]
            delete = store.deleteFromRowid('training', rowid, db_store)
            print(delete)
            sys.exit(0)

        if sys.argv[1] == 'delete-model':
            rowid = sys.argv[2]
            delete = store.deleteFromRowid('model', rowid, db_store)
            print(delete)
            sys.exit(0)


        if sys.argv[1] == 'sample-logstream':
            count = sys.argv[2]
            run = tools.sampleLogStream(count, db_store)
            sys.exit(0)

        if sys.argv[1] == 'mark-training':
            tag = sys.argv[2]
            run = store.markAllTraining(tag, db_store)
            print(run)
            sys.exit(0)

        if sys.argv[1] == 'mark-training-on':
            name = sys.argv[2]
            run = tools.markTrainingRe(name, db_store)
            sys.exit(0)

        #list-occurrence [name|-gt,-lt,-eq num]
        if sys.argv[1] == 'list-occurrence':

            try: opn = sys.argv[2]
            except IndexError: opn=None
            try: val = sys.argv[3]
            except IndexError: val=None

            if val:
                rows = store.getByOp('occurrence', opn, val, db_store)
                for row in rows:
                    print(row)
            elif opn:
                row = store.getByName('occurrence', opn, db_store)
                print(row)
            else:
                rows = store.selectAll('occurrence', db_store)
                for row in rows:
                    print(row)
            sys.exit(0)

        if sys.argv[1] == 'clear-occurrence':
            clear = store.clearAll('occurrence', db_store)
            print(clear)
            sys.exit(0)

        if sys.argv[1] == 'delete-occurrence':
            name = sys.argv[2]
            delete = store.deleteFrom('occurrence', name, db_store)
            print(delete)
            sys.exit(0)

        if sys.argv[1] == 'ip-whois':
            _ip = sys.argv[2]
            import modules.ipwhois.ipwhois

            #_ipwhois = modules.ipwhois.ipwhois.ipwhois_iplocation(_ip)
            _ipwhois = modules.ipwhois.ipwhois.ipwhois_ipapi(_ip)

            print(_ipwhois)
            sys.exit(0)

        if sys.argv[1] == 'rclnt':
            _url = sys.argv[2]
            import modules.rclnt.rclient

            _client_post = modules.rclnt.rclient.http_post(_url)

            print(_client_post)
            sys.exit(0)




        if sys.argv[1] == 'copy-occurrence':
            key = sys.argv[2]
            d={}
            from multiprocessing import shared_memory
            l = shared_memory.ShareableList(name='sentinel-shm')
            for i in range(0,len(l),2):
                _key = l[i]
                _val = l[i+1]
                if _key == key:
                    d[_key]=_val
                    break
            l.shm.close()
            l.shm.unlink()

            if key not in d.keys():
                print('Not Found: ' + key)
                sys.exit(0)

            #_data = d[key]
            #print(_data)

            data = tools.promDataParser('data', d[key])
            #print(data)

            #print('WORKING.ON')
            if str(sys.platform).startswith('linux'):
                j = { 'MESSAGE' : data }
            elif sys.platform == 'darwin':
                j = { 'eventMessage' : data }
            else:
                j = { 'data' : data }

            tag=0
            _copy = store.updateTable(tag, json.dumps(j), 'model', db_store)
            print(_copy)

            sys.exit(0)


        if sys.argv[1] == 'list-system-profile-full':
            rows = store.getAll('system_profile', db_store)
            for row in rows:
                print(row)
            sys.exit(0)

        if sys.argv[1] == 'list-system-profile':
            #name = sys.argv[2]
            #rows = store.selectAll('system_profile', db_store)
            rows = store.getAll('system_profile', db_store)
            for row in rows:
                print(row[0],row[1],row[2])
            sys.exit(0)

        if sys.argv[1] == 'gen-system-profile':
            run = tools.genSystemProfile(db_store)
            print(run)
            sys.exit(0)

        if sys.argv[1] == 'del-system-profile-name':
            name = sys.argv[2]
            delete = store.deleteFrom('system_profile', name, db_store)
            print(delete)
            sys.exit(0)

        if sys.argv[1] == 'del-system-profile-rowid':
            rowid = sys.argv[2]
            delete = store.deleteFromRowid('system_profile', rowid, db_store)
            print(delete)
            sys.exit(0)

        if sys.argv[1] == 'clear-system-profile':
            clear = store.clearAll('system_profile', db_store)
            print(clear)
            sys.exit(0)

        if sys.argv[1] == 'get-system-profile-name':
            name = sys.argv[2]
            get = store.getByName('system_profile', name, db_store)
            print(get)
            sys.exit(0)

        if sys.argv[1] == 'get-system-profile-rowid':
            rowid = sys.argv[2]
            get = store.getByID('system_profile', rowid, db_store)
            print(get)
            sys.exit(0)

        if sys.argv[1] == 'diff-system-profile-rowid':
            rowid1 = sys.argv[2]
            rowid2 = sys.argv[3]
            diff = tools.diffSystemProfileIDs(rowid1, rowid2, db_store)
            print(diff)
            sys.exit(0)

        if sys.argv[1] == 'get-system-profile-data':
            rowid = sys.argv[2]
            data = sys.argv[3]
            get = tools.getSystemProfileData(rowid, data, db_store)
            print(get)
            sys.exit(0)

        #if sys.argv[1] == 'expire-key':
        #    _key = sys.argv[2]
        #    expire = tools.setExpiregDictKeyFile(_key, db_store)
        #    print(expire)
        #    sys.exit(0)

        #if sys.argv[1] == 'expire-key':
        #    _key = sys.argv[2:]
        #    from multiprocessing import shared_memory
        #    l = shared_memory.ShareableList([_key], name='sentinel')
        #    import time
        #    time.sleep(5)
        #    l.shm.close()
        #    l.shm.unlink()
        #    sys.exit(0)

        if sys.argv[1] == 'expire-keys':
            Keys = sys.argv[2:]
            #for _key in Keys:
            #    print(_key)

            from multiprocessing import shared_memory
            l = shared_memory.ShareableList(Keys, name='sentinel-update')
            import time
            time.sleep(5)
            l.shm.close()
            l.shm.unlink()
            sys.exit(0)

        if sys.argv[1] == 'get-key':
            key = sys.argv[2]
            from multiprocessing import shared_memory
            l = shared_memory.ShareableList(name='sentinel-shm')

            for i in range(0,len(l),2):
                _key = l[i]
                _val = l[i+1]
                if _key == key:
                    print(_val.rstrip())

            l.shm.close()
            l.shm.unlink()
            sys.exit(0)

        if sys.argv[1] == 'list-keys':
            from multiprocessing import shared_memory
            l = shared_memory.ShareableList(name='sentinel-shm')
            #print(l)
            il = iter(l)
            for item in il:
                #print(item, next(il))
                print(item)
                next(il)
            l.shm.close()
            l.shm.unlink()
            sys.exit(0)

        if sys.argv[1] == 'list-vals':
            from multiprocessing import shared_memory
            l = shared_memory.ShareableList(name='sentinel-shm')
            #print(l)
            #il = iter(l)
            #for item in il:
            #    #print(value)
            #    print(next(il))
            for i in range(0,len(l),2):
                key = l[i]
                val = l[i+1]
                print(val)
            l.shm.close()
            l.shm.unlink()
            sys.exit(0)

        if sys.argv[1] == 'list-keys-metric':
            from multiprocessing import shared_memory
            l = shared_memory.ShareableList(name='sentinel-shm')
            #print(l)
            #il = iter(l)
            #for item in il:
            #    #print(item, next(il))
            #    print(item)
            #    next(il)
            #for i in range(0,len(l),2):
            #    key = l[i]
            #    val = l[i+1]
            #    #print(key)
            #    #print(val.split()[-1])
            #    print(key, val.split()[-1])
            #    #print(val.split()[-1])
            #    #print(val)

            il = iter(l)
            for item in il:
                #print(item, next(il))
                n = next(il)
                print(item, n.split()[-1])


            l.shm.close()
            l.shm.unlink()
            sys.exit(0)




        #if sys.argv[1] == 'get-shm':
        #    from multiprocessing import shared_memory
        #    l = shared_memory.ShareableList(name='sentinel-update')
        #    #print(l)
        #    for item in l:
        #        print(item)
        #    l.shm.close()
        #    l.shm.unlink()
        #    sys.exit(0)



        else:
            usage()
            sys.exit(0)
    else:
        arpTbl = tools.getArps()
        update = store.update_arp_data(db_store, arpTbl, db_manuf)
        print(update)
        sys.exit(0)


if __name__ == '__main__':
    sys.exit(main())






#!/usr/bin/env python3

import os
from subprocess import Popen, PIPE

import sys

def gitStoreLink(git_store, List, verbose=False):

    if not os.path.isdir(git_store):
        if verbose: print('mkdir ' + str(git_store))
        os.mkdir(git_store, 0o755)

    for f in List:

        gfile = git_store + f

        if not os.path.isdir(os.path.dirname(gfile)):
            if verbose: print('mkdir ' + str(os.path.dirname(gfile)))
            os.mkdir(os.path.dirname(gfile), 0o755)

        if not os.path.isfile(gfile):
            if verbose: print('link ' + str(gfile))
            os.link(f, gfile)

    return True

def gitStoreInit(git_store, verbose=False):

    if not os.path.isdir(git_store):
        if verbose: print('mkdir ' + str(git_store))
        os.mkdir(git_store, 0o755)

    if not os.path.isdir(git_store + '/.git'):
        if verbose: print('git init ' + str(git_store))
        cmd = 'git init ' + str(git_store)
        proc = Popen(cmd.split(), stdout=PIPE, stderr=PIPE)
        if verbose:
            for line in proc.stdout.readlines():
                print(line.decode('utf-8').strip('\n'))
    #os.chdir(git_store)
    return True

def gitStoreAdd(git_store, f, verbose=False):
    try:
        os.chdir(git_store)
    except FileNotFoundError as e:
        if verbose: print('FileNotFoundError: ' + str(e))
        return 'FileNotFoundError: ' + str(e)

    if not os.access(f, os.F_OK):
        if verbose: print('Not Found: ' + str(f))
        return 'Not Found: ' + str(f)
    elif not os.access(f, os.R_OK):
        if verbose: print('No Access: ' + str(f))
        return 'No Access: ' + str(f)

    cmd = 'git add ' + git_store + f
    if verbose: print('git add ' + git_store + f)

    proc = Popen(cmd.split(), stdout=PIPE, stderr=PIPE)
    stdout, stderr = proc.communicate()
    exit_code = proc.wait()

    if verbose:
        print(stdout.decode('utf-8'))
        print(stderr.decode('utf-8'))
        print(str(exit_code))

    return stdout, stderr, exit_code

def gitStoreDel(git_store, f, verbose=False):
    os.chdir(git_store)
    if verbose: print('git rm ' + git_store + f)
    cmd = 'git rm -f ' + git_store + f
    proc = Popen(cmd.split(), stdout=PIPE, stderr=PIPE)
    if verbose:
        for line in proc.stdout.readlines():
            print(line.decode('utf-8').strip('\n'))

    if os.path.exists(git_store + f):
        if verbose: print('remove ' + git_store + f)
        os.remove(git_store + f)

    git_commit = gitStoreCommit(git_store, f, verbose=True)

    return True
    

def gitStoreCommit(git_store, f, verbose=False):
    os.chdir(git_store)
    if verbose: print('git commit me ' + git_store + f)
    #import shlex
    #shlex.split(cmd)
    #cmd = 'git commit -m "sentinel" ' + git_store + f
    #proc = Popen(cmd.split(), stdout=PIPE, stderr=PIPE)

    cmd = ['git', 'commit', '-m', '"sentinel ' + str(f) + '"', git_store + f ]

    proc = Popen(cmd, stdout=PIPE, stderr=PIPE)
    stdout, stderr = proc.communicate()
    exit_code = proc.wait()

    if verbose:
        print(stdout.decode('utf-8'))
        print(stderr.decode('utf-8'))
        print(str(exit_code))

    return stdout, stderr, exit_code

def gitStoreStatus(git_store, verbose=False):
    try:
        os.chdir(git_store)
    except FileNotFoundError as e:
        if verbose: print('FileNotFoundError: ' + str(e))
        return 'FileNotFoundError: ' + str(e)

    cmd = 'git status'
    proc = Popen(cmd.split(), stdout=PIPE, stderr=PIPE)
    stdout, stderr = proc.communicate()
    exit_code = proc.wait()

    if verbose:
        print(stdout.decode('utf-8'))
        print(stderr.decode('utf-8'))
        print(str(exit_code))

    return stdout, stderr, exit_code


def gitStoreLsFiles(git_store, verbose=False):
    try:
        os.chdir(git_store)
    except FileNotFoundError as e:
        if verbose: print('FileNotFoundError: ' + str(e))
        return 'FileNotFoundError: ' + str(e)

    cmd = 'git ls-files'
    proc = Popen(cmd.split(), stdout=PIPE, stderr=PIPE)
    stdout, stderr = proc.communicate()
    exit_code = proc.wait()

    if verbose:
        print(stdout.decode('utf-8'))
        print(stderr.decode('utf-8'))
        print(str(exit_code))

    return stdout, stderr, exit_code


def gitStoreLog(git_store, verbose=False):
    try:
        os.chdir(git_store)
    except FileNotFoundError as e:
        if verbose: print('FileNotFoundError: ' + str(e))
        return 'FileNotFoundError: ' + str(e)

    cmd = 'git log'
    proc = Popen(cmd.split(), stdout=PIPE, stderr=PIPE)
    if verbose:
        for line in proc.stdout.readlines():
            print(line.decode('utf-8').strip('\n'))
    return proc.stdout.readlines()

def gitStoreClearHistory(git_store, verbose=False):
    os.chdir(git_store)

    cmd = 'git checkout --orphan temp_branch'
    proc = Popen(cmd.split(), stdout=PIPE, stderr=PIPE)
    if verbose:
        for line in proc.stdout.readlines():
            print(line.decode('utf-8').strip('\n'))

    cmd = 'git add -A'
    proc = Popen(cmd.split(), stdout=PIPE, stderr=PIPE)
    if verbose:
        for line in proc.stdout.readlines():
            print(line.decode('utf-8').strip('\n'))

    cmd = ['git','commit','-am "sentinel re-commit"']
    proc = Popen(cmd, stdout=PIPE, stderr=PIPE)
    if verbose:
        for line in proc.stdout.readlines():
            print(line.decode('utf-8').strip('\n'))

    cmd = 'git branch -D master'
    proc = Popen(cmd.split(), stdout=PIPE, stderr=PIPE)
    if verbose:
        for line in proc.stdout.readlines():
            print(line.decode('utf-8').strip('\n'))

    cmd = 'git branch -m master'
    proc = Popen(cmd.split(), stdout=PIPE, stderr=PIPE)
    if verbose:
        for line in proc.stdout.readlines():
            print(line.decode('utf-8').strip('\n'))

    #cmd = 'git push -f origin master'
    #proc = Popen(cmd.split(), stdout=PIPE, stderr=PIPE)
    #if verbose:
    #    for line in proc.stdout.readlines():
    #        print(line.decode('utf-8').strip('\n'))

    return True

#import mimetypes
#mime = mimetypes.guess_type(file)

def fileType(_file):
    try:
        with open(_file, 'r', encoding='utf-8') as f:
            f.read(4)
            return 'text'
    except UnicodeDecodeError:
        return 'binary'


def gitStoreDiff(git_store, f=None, verbose=False):
    try:
        os.chdir(git_store)
    except FileNotFoundError as e:
        if verbose: print('FileNotFoundError: ' + str(e))
        return 'FileNotFoundError: ' + str(e)

    if f is None:
        f = ''

    cmd = 'git diff ' + f

    proc = Popen(cmd.split(), stdout=PIPE, stderr=PIPE)
    stdout, stderr = proc.communicate()
    exit_code = proc.wait()

    if verbose:
        print(stdout.decode('utf-8'))
        print(stderr.decode('utf-8'))
        print(str(exit_code))

    return stdout, stderr, exit_code



if __name__ == '__main__':

    git_store = '/opt/sentinel/db/git/dir2'
    L = [ '/etc/hosts', '/etc/ssh/sshd_config' ]

    git_init = gitStoreInit(git_store)
    git_link = gitStoreLink(git_store, L)

    #for f in L:
    #    git_add  = gitStoreAdd(git_store, f)
    #    git_commit = gitStoreCommit(git_store, f)


    if sys.argv[1:]:

        if sys.argv[1] == 'git-status':
            git_status = gitStoreStatus(git_store, verbose=True)

        if sys.argv[1] == 'git-files':
            git_files = gitStoreLsFiles(git_store, verbose=True)

        if sys.argv[1] == 'git-log':
            git_log = gitStoreLog(git_store, verbose=True)

        if sys.argv[1] == 'git-add':
            _file = sys.argv[2]

            if not os.access(_file, os.F_OK):
                print('Not Found: ' + str(_file))
                sys.exit(1)
            elif not os.access(_file, os.R_OK):
                print('No Access: ' + str(_file))
                sys.exit(1)

            git_link = gitStoreLink(git_store, [_file], verbose=True)
            git_add  = gitStoreAdd(git_store, _file, verbose=True)
            git_commit = gitStoreCommit(git_store, _file, verbose=True)

        if sys.argv[1] == 'git-del':
            _file = sys.argv[2]
            git_del = gitStoreDel(git_store, _file, verbose=True)

        if sys.argv[1] == 'git-commit':
            _file = sys.argv[2]

            if not os.access(_file, os.F_OK):
                print('Not Found: ' + str(_file))
                sys.exit(1)
            elif not os.access(_file, os.R_OK):
                print('No Access: ' + str(_file))
                sys.exit(1)

            git_commit = gitStoreCommit(git_store, _file, verbose=True)


        if sys.argv[1] == 'git-clear-history':
            git_clear_hist = gitStoreClearHistory(git_store, verbose=True)

        if sys.argv[1] == 'git-diff':
            try: _file = sys.argv[2]
            except IndexError: _file = None
            git_diff = gitStoreDiff(git_store, _file, verbose=True)

        if sys.argv[1] == 'git-init':
            git_init = gitStoreInit(git_store)

        if sys.argv[1] == 'file-type':
            _file = sys.argv[2]
            file_type = fileType(_file)
            print(file_type)



# git + tegridy




#!/usr/bin/env python3

#https://gitlab.com/wireshark/wireshark/raw/master/manuf

import os
import sys

def even_up(mac):
    mac = mac.split(':')
    mac_ = mac[0]
    for i in mac[1:]:
        #print(i)
        #print(len(i))
        if len(i) == 1:
            i = '0' + i
        #print(i)
        mac_ = mac_ + ':' + i
    return mac_

def get_manufDict(db_file):
    mfDict = {}

    with open(db_file, 'r') as f:
        data = f.readlines()

    c = 0
    for line in data:
        c += 1

        if len(line.strip()) == 0:
            continue
    
        if line.startswith('#'):
            continue

        line = line.split()

        mac = line[0].lower()
        name = line[1]
        data_ = line[2:]
        data = ' '.join(data_)
        #print(mac, name, data)

        #if len(mac) == 8:
        #    mac = mac + ':00:00:00'
        if len(mac) == 20:
            mac = mac.split('/')[0]
        #print(str(len(mac)), mac)
        #print(mac)
        mfDict[mac] = name + ' (' + data + ')'

    #8 e4:1e:0a
    #20 e4:1e:0a:00:00:00/28
    return mfDict

def match_octets(mac, n, mfDict):
    #print(str(mfDict))
    #print(mac)
    mac = mac.split(':')
    n = int(n)
    matches = []
    m = mac[0]
    c = 0
    for i in mac[1:]:
        c += 1
        if c < n:
            #print(c)
            m = m + ':' + i

    #print(m)
    for k,v in mfDict.items():
        if m in k:
           #print(k,v)
           matches.append(v)
    return matches
         
def match(mac, mfDict):
    c = 2
    for i in range(0, len(mac)):
        c += 1
        #print(i, c)
        m = match_octets(mac, c, mfDict)

        #print(str(type(m)))
        #print(m)

        if len(m) == 0:
            return 'NoMatch'
        elif len(m) == 1:
            return ''.join(m)
        elif i >= 5:
            return 'MultiMatch: ' + str(m)

def get_manuf(mac, db_file):
    mac = even_up(mac.lower())
    manufDict = get_manufDict(db_file)
    manuf = match(mac, manufDict)
    return manuf


if __name__ == '__main__':
    pass

    #mac = sys.argv[1].lower()
    #mac = even_up(mac)

    #db_file = 'db/manuf'
    #manufDict = get_manufDict(db_file)

    #m = match(mac, manufDict)
    #print(m)


